--Libs
AutoLog = LibStub("AceAddon-3.0"):NewAddon("AutoLog", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("AutoLog")
local bzone = LibStub("LibBabble-Zone-3.0")
local BZ = bzone:GetUnstrictLookupTable()

--temporary fix to override the French translation for Dragon Soul
--until LibBabble-Zone is formally updated
--if GetLocale() == "frFR" then BZ["Dragon Soul"] = "Lme des dragons" end
--this version tested and working, prior version failed due to file format I suspect
if GetLocale() == "frFR" then BZ["Dragon Soul"] = "L\226\128\153\195\130me des dragons" end

--Normal mode raid list
local raid_list = {
	["ULD"] = BZ["Ulduar"],
	["NAX"] = BZ["Naxxramas"],
	["EYE"] = BZ["The Eye of Eternity"],
	["OBS"] = BZ["The Obsidian Sanctum"],
	["VOA"] = BZ["Vault of Archavon"],
	["TOC"] = BZ["Trial of the Crusader"],
	["ONY"] = BZ["Onyxia's Lair"],
	["ICC"] = BZ["Icecrown Citadel"],
	["RUS"] = BZ["The Ruby Sanctum"],
	["BWD"] = BZ["Blackwing Descent"],
	["BTW"] = BZ["The Bastion of Twilight"],
	["TFW"] = BZ["Throne of the Four Winds"],
	["BAH"] = BZ["Baradin Hold"],
	["FIR"] = BZ["Firelands"],
	["DGS"] = BZ["Dragon Soul"],
}

--Heroic mode raid list
local raid_heroic = {
	["TOC"] = BZ["Trial of the Crusader"],
	["ICC"] = BZ["Icecrown Citadel"],
	["RUS"] = BZ["The Ruby Sanctum"],
	["BWD"] = BZ["Blackwing Descent"],
	["BTW"] = BZ["The Bastion of Twilight"],
	["TFW"] = BZ["Throne of the Four Winds"],
	["FIR"] = BZ["Firelands"],
	["DGS"] = BZ["Dragon Soul"],
}

--LFR mode raid lsit
local raid_lfr = {
	["DGS"] = BZ["Dragon Soul"],
}

--Option table
local options = {
	name = "AutoLog",
	handler = AutoLog,
	type = "group",
	args = {
		enable = { -- Global enable key
			type = "toggle",
			name = L["Enable auto logging"],
			desc = L["If unchecked, AutoLog will never enable Combat Logging"],
			get = "IsALenabled",
			set = "ToggleEnable",
			width = "double",
			order = 1,
		},
		dungeons = { -- Heroic dungeons
			type = "toggle",
			name = L["Log 5 player heroic instances"],
			set = "SetDungeons",
			get = "GetDungeons",
			order = 2,
			width = "full",
		},
		curLogging = { -- Logging indicator
			type = "toggle",
			name = L["Logging state"],
			disabled = true,
			desc = L["Checked if combat logging is currently on"],
			get = "IsCurrentlyLogging",
			set = "setCurrLog",
			order = 3,
		},
		raids10 = { -- 10 Normal raids select
			type = "multiselect",
			name = L["10 Players Raids"],
			desc = L["10 Players raid instances where you want to log combat"],
			values = raid_list,
			tristate = false,
			set = "SetRaid10",
			get = "GetRaid10",
			order = 4,
		},
		raids25 = { -- 25 Normal raids select
			type = "multiselect",
			name = L["25 Players Raids"],
			desc = L["25 Players raid instances where you want to log combat"],
			values = raid_list,
			tristate = false,
			set = "SetRaid25",
			get = "GetRaid25",
			order = 5,
		},
		raids10h = { -- 10 Heroic raids select
			type = "multiselect",
			name = L["10 Players Heroic Raids"],
			desc = L["10 Players heroic raid instances where you want to log combat"],
			values = raid_heroic,
			tristate = false,
			set = "SetRaid10h",
			get = "GetRaid10h",
			order = 6,
		},
		raids25h = { -- 25 Heroic raids select
			type = "multiselect",
			name = L["25 Players Heroic Raids"],
			desc = L["25 Players heroic raid instances where you want to log combat"],
			values = raid_heroic,
			tristate = false,
			set = "SetRaid25h",
			get = "GetRaid25h",
			order = 7,
		},
		lfr = { -- LFR raids select
			type = "multiselect",
			name = L["LFR Raids"],
			desc = L["Raid finder instances where you want to log combat"],
			values = raid_lfr,
			tristate = false,
			set = "SetRaidLFR",
			get = "GetRaidLFR",
			order = 8,
		},
	},
}

--Default DB
local defaults = {
	char = {
		enable = false,
		curLogging = false,
		dungeons = false,
		raids10 = {
			["ULD"] = false,
			["NAX"] = false,
			["EYE"] = false,
			["OBS"] = false,
			["VOA"] = false,
			["TOC"] = false,
			["ONY"] = false,
			["ICC"] = false,
			["RUS"] = false,
			["BWD"] = false,
			["BTW"] = false,
			["TFW"] = false,
			["BAH"] = false,
			["FIR"] = false,
			["DGS"] = false,
		},
		raids25 = {
			["ULD"] = false,
			["NAX"] = false,
			["EYE"] = false,
			["OBS"] = false,
			["VOA"] = false,
			["TOC"] = false,
			["ONY"] = false,
			["ICC"] = false,
			["RUS"] = false,
			["BWD"] = false,
			["BTW"] = false,
			["TFW"] = false,
			["BAH"] = false,
			["FIR"] = false,
			["DGS"] = false,
		},
		raids10h = {
			["TOC"] = false,
			["ICC"] = false,
			["RUS"] = false,
			["BWD"] = false,
			["BTW"] = false,
			["TFW"] = false,
			["FIR"] = false,
			["DGS"] = false,
		},
		raids25h = {
			["TOC"] = false,
			["ICC"] = false,
			["RUS"] = false,
			["BWD"] = false,
			["BTW"] = false,
			["TFW"] = false,
			["FIR"] = false,
			["DGS"] = false,
		},
		lfr = {
			["DGS"] = false,
		}
	},
}

--Init function : Creates DB / chat commands
function AutoLog:OnInitialize()

	LibStub("AceConfig-3.0"):RegisterOptionsTable("AutoLog", options)
	self.db = LibStub("AceDB-3.0"):New("AutoLog_DB", defaults)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AutoLog", "AutoLog v" .. GetAddOnMetadata("AutoLog", "Version"))
	self:RegisterChatCommand("autolog", "ChatCommand")
	self:RegisterChatCommand("al", "ChatCommand")

end

--Enable function : Register event
function AutoLog:OnEnable()

	self:Print("Version " .. GetAddOnMetadata("AutoLog", "Version") .. L[" loaded."])
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	
	self:CheckLog()

end

--Disable function ... should not be called, but must exist
function AutoLog:OnDisable()

	self:Print(L["AutoLog Disabled"])
	self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	
end

--Event handler
function AutoLog:ZONE_CHANGED_NEW_AREA()
	
	self:CheckLog()

end

function AutoLog:PLAYER_ENTERING_WORLD()

	self:CheckLog()
	
end

--check for Fall of Deathwing component of Dragon Soul
--looks for a raid group in The Maelstrom
local function checkFOD(area)

	local inRaid = (select(2, GetInstanceInfo()) == "raid")
	local inFOD = false
	
	if inRaid and (area == strlower(BZ["The Maelstrom"])) then inFOD = true end
	return inFOD 
	
end

--GetKey function :
--		- Returns the key name of the raid, if it exists, false otherwise
function AutoLog:GetKey(t, area)

	local k, v
	local rk = false
	
	if checkFOD(area) then rk = "DGS"	
	else
		for k, v in pairs (t) do
			--self:Print(k .. " : " .. v)
			if strlower(area) == strlower(v) then
				rk = k
				break
			end
		end
	end

	return rk

end

--based on GetCurrentInstanceDifficulty from Deadly Boss Mods
function AutoLog:GetCurrentInstanceDifficulty()

	local _, instanceType, difficulty, _, maxPlayers = GetInstanceInfo()
	
	if IsPartyLFG() and IsInLFGDungeon() and difficulty == 2 and instanceType == "raid" and maxPlayers == 25 then return 5
	else return difficulty
	end

end

--CheckLog function :
--		- Called on event / option changes
--		- Checks whether combat logging should be enabled (or not)
function AutoLog:CheckLog()

	local area = GetRealZoneText()
	--local difficulty = GetInstanceDifficulty()
	local difficulty = self:GetCurrentInstanceDifficulty()
	local isLogging = LoggingCombat()
	local isDungeon = (select(2, GetInstanceInfo()) == "party")
	local raidKey, raidType
	local goLog = false
	
	if not self.db.char.enable then return end -- Add-on is disabled

	print("Current area : ",area)
	--print("Difficuly : ",difficulty)
	--fix to prevent an error occuring if area is nil
	if not area then return
	else area = strlower(area) end
	
	raidKey = self:GetKey(raid_list, area)
	
	if difficulty == 1 then raidType = "raids10"
	elseif difficulty == 2 then raidType = "raids25"
	elseif difficulty == 3 then raidType = "raids10h"; raidKey = self:GetKey(raid_heroic, area)
	elseif difficulty == 4 then raidType = "raids25h"; raidKey = self:GetKey(raid_heroic, area)
	elseif difficulty == 5 then raidType = "lfr"; raidKey = self:GetKey(raid_lfr, area) end

	if ((self.db.char.dungeons) and (isDungeon)) then goLog = true		
	elseif raidKey then goLog = self.db.char[raidType][raidKey] end
	
	if goLog and not isLogging then self:Print(L["Combat logging turned on"])
	elseif isLogging and not goLog then self:Print(L["Combat logging turned off"]) end

	self.db.char.curLogging = LoggingCombat(goLog)
	
end

function AutoLog:ChatCommand(input)

	if not input or input:trim() == "" then

		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)

	else

		LibStub("AceConfigCmd-3.0").HandleCommand(AutoLog, "autolog", "AutoLog", input)

	end

end

function AutoLog:SetCurrLog(info,newValue)

	self.db.char.curLogging = newValue

end

function AutoLog:IsCurrentlyLogging(info)

	self.db.char.curLogging = LoggingCombat()
	return self.db.char.curLogging

	end

function AutoLog:IsALenabled(info)

	return self.db.char.enable

	end

function AutoLog:ToggleEnable(info, newValue)

	self.db.char.enable = newValue
	self:CheckLog()

end

function AutoLog:GetRaid10(info,raid)

	return self.db.char.raids10[raid]

end

function AutoLog:SetRaid10(info,raid,val)

	self.db.char.raids10[raid] = val
	self:CheckLog()

end

function AutoLog:GetRaid25(info,raid)

	return self.db.char.raids25[raid]

end

function AutoLog:SetRaid25(info,raid,val)

	self.db.char.raids25[raid] = val
	self:CheckLog()

end

function AutoLog:GetRaid10h(info,raid)

	return self.db.char.raids10h[raid]

end

function AutoLog:SetRaid10h(info,raid,val)

	self.db.char.raids10h[raid] = val
	self:CheckLog()

end

function AutoLog:GetRaid25h(info,raid)

	return self.db.char.raids25h[raid]

end

function AutoLog:SetRaid25h(info,raid,val)

	self.db.char.raids25h[raid] = val
	self:CheckLog()

end

function AutoLog:GetDungeons()

	return self.db.char.dungeons

end

function AutoLog:SetDungeons(info, val)

	self.db.char.dungeons = val
	self:CheckLog()

end

function AutoLog:GetRaidLFR(info, raid)

	return self.db.char.lfr[raid]
	
end

function AutoLog:SetRaidLFR(info, raid, val)

	self.db.char.lfr[raid] = val
	self:CheckLog()
	
end